/*
 * This program SHOULD run in an ESP8266 module directly attached to a "mouse" robot.
 * The plan is for it to receive UDP packets that control the speeds of its two wheels.
 *
 * The received packet should contain two three-byte integers in the range of 0-999 with
 * a comma between. The first controls the left wheel's PWM duty cycle and the second
 * controls the right wheel's duty cycle.
 *
 * Any packets with the wrong length OR the comma missing will be printed on the serial
 * connection but otherwise completely ignored.
*/

#include <ESP8266WiFi.h>
#include <WiFiUDP.h>

// Includes for running display
#include <SPI.h>
#include <U8g2lib.h>
char forDisp[64];  // buffer for building display items

WiFiUDP Udp;

const char* ssid     = "HomeToo";
const char* password = "WiFiPass";

#define LEFT_M 16     // Left motor connection
#define RIGHT_M 14    //  Right motor connection
#define UDPPORT 4949

// where is the display attached? HERE
#define CLOCK_GPIO 5
#define DATA_GPIO 4
/* Constructor */
// This F version keeps a full buffer in RAM and uses sendBuffer() to display
U8G2_SSD1306_128X64_NONAME_F_SW_I2C display(U8G2_R0, CLOCK_GPIO, DATA_GPIO); // for 1.3" IIC

void setup() {
    // ready screen for display
    display.begin();

    Serial.begin(115200);
    delay(10);

    Serial.println();
    Serial.print("Connecting to ");
    Serial.println(ssid);

    WiFi.begin(ssid, password);

    while(WiFi.status() != WL_CONNECTED) {
      Serial.print(".");
        delay(500);
    }

    Serial.println("\nWiFi connected");
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());

    display.clearBuffer();
    display.setFont(u8g2_font_pxplusibmvga9_tr); // just fits IP on screen
    // display.setFont(u8g2_font_ncenB12_tf);   // too big, does not fit
    // display.setFont(u8g2_font_6x10_tf);      // pretty small but works
    sprintf(forDisp, "SSID %s", ssid);
    display.drawStr(2, 20, forDisp);
    sprintf(forDisp, "%s", WiFi.localIP().toString().c_str());
    display.drawStr(2, 50, forDisp);
    display.sendBuffer();
    delay(50);

    if(Udp.begin(UDPPORT) == 1){
      Serial.println("UDP connected");
      // initialize pins
      analogWrite(LEFT_M,1023);
      analogWrite(RIGHT_M,1023);
      } else {
      Serial.println("UDP NOT connected");
      }
}

#define PKTSZ 30
void loop() {
  int packetSize, l_speed, r_speed;
  char inPac[PKTSZ+1];     // allow room for 0x00 to terminate string
  packetSize = Udp.parsePacket();
  if(packetSize > 0 ) {
    Serial.println("");
    Serial.print("Received packet of size ");
    Serial.println(packetSize);

    packetSize = Udp.read(inPac, PKTSZ);
    inPac[packetSize]=0;
    Serial.println(inPac);
    if (packetSize == 7 && inPac[3]==',') {
      l_speed = (inPac[0]-'0')*100 + (inPac[1]-'0')*10 + inPac[2]-'0';
      r_speed = (inPac[4]-'0')*100 + (inPac[5]-'0')*10 + inPac[6]-'0';
      Serial.print("Left speed=");
      Serial.print(l_speed);
      Serial.print("  Right speed=");
      Serial.println(r_speed);
      analogWrite(LEFT_M,1023-l_speed);
      analogWrite(RIGHT_M,1023-r_speed);

      display.clearBuffer();
//      display.setFont(u8g2_font_ncenB12_tf);   // works, but small
//      display.drawStr(2, 20, inPac);
//      display.setFont(u8g2_font_ncenB14_tr);   // works, bigger
//      display.drawStr(2, 50, inPac);
      display.setFont(u8g2_font_ncenB24_tr);     // works, fills screen center
      display.drawStr(2, 60, inPac);
      display.setFont(u8g2_font_pxplusibmvga9_tr); // just fits IP on screen
      sprintf(forDisp, "%s", WiFi.localIP().toString().c_str());
      display.drawStr(2, 15, forDisp);
      display.sendBuffer();
    }
  }
  delay(50);
}
